$subfile Ex1   $c----------- Example 1 -----------------------

               A simple linear model with qualitative factors.

               We analyse an artificially constructed 2-factor
               cross-classification with 2 replicates in each
               cell.  The data are:

                   COLUMN  1       2       3       4
                 ROW
                  1      18.6    16.4    14.9    20.0
                         18.4    16.2    15.1    19.8

                  2      12.9    10.9    10.1    14.4
                         13.1    11.1     9.9    14.6

               In fact each observation has been calculated as
               the sum of known row, column and interaction
               effects, and 'errors' of plus/minus 0.1

               Clearly we would hope to recover this structure from
               the data.  We also show how to omit a point from a fit
               and how to fit an alternative error structure in which
               the standard error is proportional to the mean.

$c             There are 16 observations, and we declare this by:
$units 16

$c             We read the observations in free format
$data OBS   $read
18.6 18.4  16.4 16.2  14.9 15.1  20.0 19.8
12.9 13.1  10.9 11.1  10.1  9.9  14.4 14.6

$c             We could also read in the values of ROW, COLUMN
               and REPLICATE but as they are in standard order
               it is easier to generate them
$calculate ROW=%gl(2,8) : COLUMN=%gl(4,2) : REPLICATE=%gl(2,1) $
$c             We also declare them as factors as their values
               represent level names not real numbers
$factor  ROW 2  COLUMN 4  REPLICATE 2
$c             The data can now be inspected, either as a data
               matrix
$look OBS ROW COLUMN REPLICATE $
$c             or as a table
$tprint  OBS  ROW;REPLICATE;COLUMN $

$c             We declare the y-variate
$yvariate  OBS
$c             The null fit gives the total sum of squares
$fit 1 $
$c             We now try the effects of ROW,COL and their interaction
$fit ROW : +COLUMN : +ROW.COLUMN $
$c             If individual effects are fitted, i.e. with a
               parameter for each observation, the deviance
               will drop to zero
$fit +ROW.COLUMN.REPLICATE $
$c             Or near enough zero.  A complete model like this
               is of no practical interest.
               We return to the ROW*COLUMN model and display
               parameter estimates, fitted values,etc.
$fit -ROW.COLUMN.REPLICATE   $display e r $

$c             To try the effect of omitting the last point with
               an additive model, create a weight vector and refit.
$calc WEIGHT=1  :  WEIGHT(16)=0   $weight WEIGHT
$c             The above warning indicates that the previous
               model has been changed (by setting a weight
               vector), so the DISPLAY directive cannot be
               used until after the next fit.
$fit ROW+COLUMN $
$c             Compare this with the deviance of 0.3 with 11 df
               obtained above.  The effect of omitting the
               point is small, as would be expected.

$c             Now restore the point and try a fit with constant
               proportional standard error.
$weight   $error gamma   $link identity
$fit ROW+COLUMN $
$c             To compare this deviance directly with the fit
               assuming normal errors, multiply it by the
               square of the grand mean of the observations.
               (Notice the use of system scalar %dv holding
               the deviance)
$tabulate the OBS mean into OBS_MEAN $
$cal %dv*OBS_MEAN**2 $
$c             The fits are indistinguishable.


$delete COLUMN ROW REPLICATE OBS WEIGHT OBS_MEAN
$c   end of Ex. 1




$subfile Ex2   $c---------- Example 2 ---------------------------

               Analysis of covariance with grouped binomial data.

               In this example we have in effect a simple analysis
               of covariance with 2 groups of data, except that the
               response, instead of being a linear function of the
               covariate plus Normally-distributed error, is
               binomially distributed with its probability depending
               on the covariate through the logistic function.

$c             There are 12 observations with a 2-level grouping
               factor
$units 12 $factor GROUP 2
$c             (The printed warning reminds you that the model has
               been reset after the UNITS directive.)
               Set up and read the data for GROUP, the covariate X
               and binomial response, R out of N
$data GROUP X R N   $READ
 1 2.6 34  42    1 3.8 51  52    2 0.3 31 115
 1 1.2  4  28    2 5.8 48  56    2 3.0 55  88
 1 0.1  2  64    2 7.8 71  75    1 1.8 13  32
 2 4.4 31  39    2 1.5 47 104    1 3.3 16  18

$c             Set up the model
$yvariate R $error binomial N
$c             Model now set up.  Logit link declared by default
$fit : GROUP $
$c             Group differences are small.  Try the covariate,
               ignoring grouping, to see if a single line fits
$fit X $
$c             Keep a common slope, but try different intercepts
$fit +GROUP $
$c             Fit still bad, so try different slopes
$fit +GROUP.X $
$c             Fit now good, so check if common intercept
               is adequate
$fit -GROUP $
$c             It is not, so restore separate intercepts and
               display results
$fit +GROUP   $display  e r $

$c             We can also plot the fitted proportions using
               different symbols to distinguish the groups.
               Note the use of the system vector %FV holding the
               fitted values.
$calc FIT=%FV/N $
$plot FIT  X  '+o' GROUP $
$c             Note that the fitted proportions do not lie on a
               straight line as it is the logit of the proportions
               that is linear in the covariate.


$delete GROUP X R N FIT
$c   end of Ex. 2



$subfile Ex3   $c---------- Example 3 --------------

               Fisher's data on tuberculin response.

               Fisher ('A biological assay of tuberculins',
               Biometrics, 1949, p300-316) published data consisting
               of 16 measurements of TUBERCULIN response to four
               TREATMENTS which we call 1, 2, 3 and 4.  These were
               applied in a Latin sqare design so that the effects
               were not confounded with type of COW and SITE.

               The data can be read in as follows:

$units 16   $factor COW 4 SITE 4 TREATMENT 4
$data TUBERCULIN COW SITE TREATMENT    $read
 454 1 1 1
 408 1 2 2
 523 1 3 3
 364 1 4 4
 249 2 1 2
 322 2 2 1
 268 2 3 4
 283 2 4 3
 349 3 1 3
 312 3 2 4
 411 3 3 1
 266 3 4 2
 249 4 1 4
 347 4 2 3
 285 4 3 2
 290 4 4 1

$c             The data can be displayed by the TPRINT directive
               in a 2-way layout in which TUBERCULIN and TREATMENT
               are tabulated in parallel by SITE and COW
$tprint  TUBERCULIN;TREATMENT  SITE;COW  $

$c             The TREATMENTS themselves could be considered as
               a 2-by-2 factorial arrangement indexed by 2 factors,
               which we shall call A and B, in the following way:

                    TREATMENT(1) = A(1) with B(1)
                    TREATMENT(2) = A(2) with B(1)
                    TREATMENT(3) = A(1) with B(2)
                    TREATMENT(4) = A(2) with B(2)

               We can recode into A and B by:
$assign TA=1,2,1,2   $group  A=TREATMENT   values TA  $
$assign TB=1,1,2,2   $group  B=TREATMENT   values TB  $
$c             and print as a table:
$tprint TUBERCULIN;A;B  SITE;COW  $

$c             Fisher believed that the variances of the observations
               were proportional to their expectations and that the
               systematic part of the model was linear on the
               log-scale, indicating the use of the log link with
               Poisson errors, but with unknown scale parameter.
               We set this by
$yvariate TUBERCULIN  $error poisson   $scale   $
$c             Note that the log link is set by default.

$c             We do a null fit to get idea of variance about the
               overall mean
$fit 1 $
$c             Then we remove the COW and SITE effects
$fit +COW+SITE $
$c             Now we try treatment B
$fit +B $
$c             Now we try A also
$fit +A $
$c             The B term does not reduce the deviance significantly.
               We display the estimates, fitted values etc. and
               correlations of estimates for current model
$display merc
$c             The low correlation between A and B (parameters
               8 and 9) means that they would have given very
               similar changes in deviance if fitted in the
               opposite order.


$c             Another possible approach would be to stabilise
               the variance by taking square roots,treat the
               error distribution as normal, the systematic
               effect still being linear on the log scale.
$calc TUBERCULIN=2*%sqrt(TUBERCULIN)   $
$c             The factor 2 makes the variance of the new
               TUBERCULIN approximately the same as the variance/mean
               ratio of the old.
$error normal  $link logarithm
$fit 1 $
$fit +COW+SITE $
$fit +B $
$fit +A $
$c             Note how closely the deviances agree with those
               obtained previously.
$display mer
$c             Notice how closely the estimates (except that of
               the constant 1) agree with the previous ones if we
               double them (we should really have had the link as
               2*log(mu) ).


$delete  TUBERCULIN TREATMENT A B COW SITE TA TB
$c   end of Ex. 3



$subfile Ex4   $c---------- Example 4 ---------------------

               The use of macros in model fitting.

               The data are taken from Bliss ('Statistics in Biology',
               1970, Vol II) Table 15.21.  COMPLEMENT is the
               concentration of complement producing 50% haemolysis
               of sheep erythrocytes at concentrations of a
               haemolytic antiserum ANTISERUM.


$units  9   $data ANTISERUM COMPLEMENT   $read
 0.50 10.22 0.75 7.37 1.00 5.72 1.50 4.78 2.00
 4.30  3.00 3.85 4.00 3.74 6.00 3.54 8.00 3.39
$c             Plot data to get some idea of functional form
$plot COMPLEMENT ANTISERUM $
$c             Roughly hyperbolic, so set up family
               of inverse transformations for ANTISERUM  with an
               unknown origin, denoted by the scalar %d
$macro CYCLE $calc HYPER=1/(ANTISERUM+%D) $fit . $$e
$c             Note use of . to repeat same model.
               Search for a value of scalar %d that minimises
               the deviance, starting at zero.
$error gamma  $yvariate COMPLEMENT
$cal HYPER=1/ANTISERUM  $fit HYPER $
$c             Now use macro CYCLE for trial values of %d
$cal %d=1    $use CYCLE $
$cal %d=.5   $use CYCLE $
$cal %d=.55  $use CYCLE $
$cal %d=.51  $use CYCLE $
$cal %d=.53  $use CYCLE $
$cal %d=.52  $use CYCLE $
$c             An accurate enough value.  Now display the details of
               the fit, remembering that standard errors do not allow
               for the estimation of %d and that the d.f. of the
               deviance should be reduced by 1.
$display  m e r
$c             Inspect the fit by forming normalised residuals
               and plotting them against an approximation
               to the order statistics of a normal sample
$calc RES=(%YV**(1/3)-%FV**(1/3))/%FV**(1/3)
  : O_RES=%nd((%cu(1)-0.5)/9)
$sort RES   $plot RES O_RES  'o' $

$c             Compare this with Bliss's original fit
               using normal errors and %d=.4572
$error normal  $link  reciprocal  $calc  %d=.4572  $use CYCLE $
$display  e r
$calc RES=%YV-%FV   $sort RES   $plot(style=1)  RES  O_RES  'o'  $
$c             The fit is markedly less good.  Gamma errors gave a
               better model.  Finally try using a prior value
               for the asymptote.  Try an intercept of 1/3 on
               inverse scale
$calc INT=1/3   $offset INT
$c             Use original model with best %d
$error gamma   $calc %d=.52   $fit HYPER-1   $dis er
$c             Compare estimate from the data of .3212 as
               against .3333333... used here, and note pattern
               in the residuals from use of latter.


$delete COMPLEMENT ANTISERUM HYPER CYCLE RES O_RES INT
$c   end of Ex. 4



$subfile Ex5     $c---------- Example 5 --------------------

                 Some simple macros for data manipulation.

                 This example shows how GLIM can be used to program
                 macros for simple data manipulation - we want to take
                 an existing vector and either expand or contract it
                 into a new vector.  Note the use of argument
                 passing via character strings.


$mac COLLAPSE
$c Assigns to a shorter vector %1 the elements of a longer
   vector %2 for which a condition in macro %3 is true.
   N.b.  %z1 is a scalar used for temporary storage.
$cal %z1 = %cu(#%3)   $var  %z1 %1
$cal %1( #%3 * %cu(#%3) ) = %2  $$end


$mac STRETCH
$c  Assigns to a longer vector %1 the elements of a
    shorter vector %2 when the condition in macro
    %3 is true.
$cal %1 = %2( #%3 * %cu(#%3) )  $$end


$c               Example of use of COLLAPSE.  Suppose vectors
                 A, B and X have values as follows

$assign  A =  1,    2,    1,    2,    1,    2,    1,    2
   :     B =  1,    2,    3,    4,    5,    6,    7,    8
   :     X = 1.5,  2.3,  3.7,  4.8,  5.2,  6.4,  7.9,  8.1   $

$c               and it is required to extract into Y the
                 values of X for which A is equal to 2 and
                 B is greater than 3.

$c               We can define the condition by
$mac CONDITION    ( A==2 & B>3 )  $end
$c               and set arguments and activiate the macro by
$use COLLAPSE   Y  X  CONDITION   $
$c               The effect can be checked by
$print  Y   $


$c               The instruction
$use STRETCH   Z  Y  CONDITION   $
$c               will cause Z to have the values of Y where
                 CONDITION is true and zero elsewhere.
$print  Z   $

$c               Thus the joint effect of COLLAPSE and STRETCH
$use COLLAPSE  Y X CONDITION   $use  STRETCH  Z Y CONDITION  $
$c               is equivalent to
$calc R = X * #CONDITION
$c               which can be checked by
$look  Z R  $


$delete  STRETCH CONDITION COLLAPSE A B X Y Z R
$c   end of Ex. 5



$subfile Ex6   $c---------- Example 6 ----------

               Simulation and output control.

               Data are first simulated for a randomised
               block experiment with 4 blocks and 5
               treatments, using an additive model with
                  grand mean   20
                  block effects       -4 -2  5  1
                  treatment effects   -3  0  4 -2  1
               and an error term generated from the standard
               pseudo-random number generator %sr.


$units 20    $factor BLOCK 4 TREAT 5
$cal BLOCK=%gl(4,5) : TREAT=%gl(5,1)
$assign B = -4, -2,  5,  1
$assign T = -3,  0,  4, -2,  1
$cal Y=20+B(BLOCK)+T(TREAT)+%nd(%sr(0))

$c             Print out the values obtained
$tprint(style=1)    Y   BLOCKS;TREAT  $



$c             The next section fits successively the
               models 1, +BLOCK, +TREAT, switching off the
               standard GLIM output, and uses the deviances
               and d.f. obtained to construct the standard
               S.S. and d.f. of the Analysis of Variance
               table.  The M.S. are also formed and all are
               then embedded in text to produce a line of
               that table.

$macro ANOVA
$yvariate  %1
$print ;;; '         Analysis of Variance' ; ;
' Component         S.S.    D.F.   M.S. ' ;
$calc %z1=%coc : %z2=%col : %z3=%coh
$output  $trans  !  switch off output

$fit  ! grand mean only
$calc %s=%dv : %d=%df : %a=%s:%b=%d

$fit %2  ! add in blocks
$use LINE TXBLOCK

$fit +%3  ! then treatments
$use LINE TXTREAT

$calc %dv=0 : %df=0
$use LINE TXERROR

$output %z1 %z2 %z3  $trans i w f h o
$calc %m=%a/%b
$print ; '  Total       '
  *real %a,9,2   *integer %b,6   *real %m,9,2  ;;;; $

$end


$macro LINE !
$calc %s=%s-%dv : %d=%d-%df : %m=%s/%d
$output %z1 %z2 %z3  $trans i w f h o
$print     %1
  *real %s,9,2   *integer %d,6   *real %m,9,2
$output  $trans
$cal %s=%dv :%d=%df $$end

$macro TXBLOCK   Blocks      $end
$macro TXTREAT   Treatments  $end
$macro TXERROR   Error       $end


$c             Invoke the macro
$use ANOVA   Y BLOCK TREAT  $


$delete  BLOCK TREAT B T Y LINE TXBL TXTR TXER ANOVA
$c   end of Ex. 6


$subfile Ex7   $c---------- Example 7 -------------

$c             Tabulation and table printing.

               The following examples illustrate some uses
               of the TABULATE and TPRINT directives, using
               simulated data.


$c             Generate random variates and factors
$units 100
$cal Y = 100*%sr(0) $
$fac GROUP 5  $cal GROUP=%sr(4)+1 $
$cal AGE=60+%sr(4)  $
$cal WT=%sr(0)  $
$fac SEX 2    $cal SEX=%sr(1)+1  $
$fac CLASS 4  $cal CLASS=%sr(3)+1  $



$c             Various statistics can be calculated from the values
               held in a vector:
$tabulate  the Y mean  $
$tabulate  the Y percentile 60  $



$c             Group means are formed by:
$tabulate  the Y mean  for GROUP  $



$c             The results can be stored:
$tabulate  the Y mean  for GROUP  into GMEAN  $
$c             and printed:
$tprint GMEAN 5 $



$c             If variates are used the table is classified by the
               distinct values that occur in the vector.
$tabulate  the Y mean  for AGE  $



$c             Classifying vectors are generated using the BY option.
$tabulate  the Y mean  for GROUP;AGE
           into MEAN   by CGRP;CAGE    $
$tprint   MEAN CGRP;CAGE  $



$c             Weights are specified by the WITH option.
$tabulate  the Y mean  with WT  for GROUP  $
$c             The weights can be stored by the USING option
$tabulate  the Y mean  with WT    for GROUP
                       using SWT  by GRP     $
$tprint SWT GRP $
$c             The default weight is 1 so that frequencies can be
               calculated as follows
$tabulate  for AGE   using FREQ  by AGE2  $
$tprint FREQ AGE2 $
$c             Output weights can be printed using an output request
$tabulate  for AGE   using []  $



$c             Contingency tables are formed in similar fashion
$tabulate  for AGE;GROUP  using CTAB  $
$tprint  CTAB CAGE;CGRP  $
$c             Collapsing onto a margin can be achieved by either...
$tabulate  with CTAB  for CGRP   using []   $
$c             ...or
$tabulate  the CTAB total  for CGRP  $



$c             Multiple tables can be printed in parallel.
$tabulate  the Y mean  with WT    for AGE;GROUP
           into MEAN   using MWT  by CAGE;CGRP   $
$tprint  MEAN;MWT  CAGE;CGRP  $
$c             Multi-way tables are handled similarly
$tabulate  for GROUP;SEX;CLASS   using TABLE  by IGRP;ISEX;ICLASS  $
$tprint  TABLE  IGRP;ISEX;ICLASS  $



$c             Models can be specified using the generated
               classification vectors
$units 40
$yvar TABLE   $error poisson   $fit IGRP+ISEX+ICLASS $
$tprint  %yv;%fv  IGRP;ISEX;ICLASS  $



$delete Y GROUP AGE WT SEX CLASS GMEAN MEAN CGRP
CAGE SWT GRP FREQ CTAB AGE2 MWT TABLE IGRP ISEX ICLASS
$c   end of Ex. 7


$stop
$finish
