      SUBROUTINE DQRST(X,LDX,N,P,Y,TOL,B,RSD,K,JPVT,QRAUX,WORK)
      INTEGER LDX,N,P,K,JPVT(1)
      DOUBLE PRECISION X(LDX,1),Y(1),TOL,B(1),RSD(1),QRAUX(1),WORK(1)
C
C     DQRST IS A SUBROUTINE TO COMPUTE LEAST SQUARES SOLUTIONS
C     TO THE SYSTEM
C
C     (1)               X * B = Y,
C
C     WHICH MAY BE EITHER UNDER-DETERMINED OR OVER-DETERMINED.
C     THE USER MAY SUPPLY A TOLERANCE TO LIMIT THE COLUMNS OF
C     X USED IN COMPUTING THE SOLUTION.  IN EFFECT, A SET OF
C     COLUMNS WITH A CONDITION NUMBER APPROXIMATELY BOUNDED BY
C     1/TOL IS USED, THE OTHER COMPONENTS OF B BEING SET TO ZERO.
C
C     ON ENTRY
C
C        X     DOUBLE PRECISION(LDX,P), WHERE LDX.GE.N
C              X CONTAINS THE NXP COEFFICIENT MATRIX OF
C              THE SYSTEM (1), X IS DESTROYED BY DQRST.
C
C        LDX   INTEGER
C              LDX IS THE LEADING DIMENSION OF THE ARRAY X.
C
C        N     INTEGER
C              N IS THE NUMBER OF ROWS OF THE MATRIX X.
C
C        P     INTEGER
C              P IS THE NUMBER OF COLUMNS OF THE MATRIX X.
C
C        Y     DOUBLE PRECISION(N)
C              Y CONTAINS THE RIGHT HAND SIDE OF THE SYSTEM (1).
C
C        TOL   DOUBLE PRECISION
C              TOL IS THE NONNEGATIVE TOLERANCE USED TO
C              DETERMINE THE SUBSET OF COLUMNS OF X INCLUDED
C              IN THE SOLUTION. IF TOL IS ZERO, A FULL
C              COMPLEMENT OF MIN(N,P) COLUMNS IS USED.
C
C        JPVT  INTEGER(P)
C              JPVT IS AN ARRAY USED BY DQRDC.
C
C        QRAUX  DOUBLE PRECISION(P)
C               QRAUX IS AN ARRAY USED BY DQRDC AND DQRSL.
C
C        WORK   DOUBLE PRECISION(P)
C               WORK IS A WORK ARRAY USED BY DQRDC.
C
C     ON RETURN
C
C        X      X CONTAINS THE OUTPUT ARRAY FROM DQRDC.
C
C        B      DOUBLE PRECISION(P)
C               B CONTAINS THE SOLUTION VECTOR. COMPONENTS
C               CORRESPONDING TO COLUMNS NOT USED ARE SET TO ZERO.
C
C        RSD    DOUBLE PRECISION(N)
C               RSD CONTAINS THE RESIDUAL VECTOR Y-X*B.
C
C        K      INTEGER
C               K CONTAINS THE NUMBER OF COLUMNS USED IN THE
C               SOLUTION.
C
C        JPVT   CONTAINS THE PIVOT INFORMATION FROM DQRDC.
C
C        QRAUX  CONTAINS THE ARRAY OUTPUT BY DQRDC.
C
C     ON RETURN THE ARRAYS X, JPVT AND QRAUX CONTAIN THE
C     USUAL OUTPUT FROM DQRDC, SO THAT THE QR DECOMPOSITION
C     OF X WITH PIVOTING IS FULLY AVAILABLE TO THE USER.
C     IN  PARTICULAR,  COLUMNS  JPVT(L),  JPVT(2),...,JPVT(K)
C     WERE USED IN THE SOLUTION,  AND  THE  CONDITION  NUMBER
C     ASSOCIATED WITH THOSE COLUMNS IS ESTIMATED BY
C     ABS(XTL,L)/X(K,K)).
C
C     DQRST USES THE LINPACK SUBROUTINES DQRDC AND DQRSL.
C
C     INTERNAL VARIABLES.
C
      INTEGER INFO,J,KK,M
      DOUBLE PRECISION T
C
C     INITIALIZE JPVT SO THAT ALL COLUMNS ARE FREE.
C
      DO 10 J=1,P
      JPVT(J) = 0
   10 CONTINUE
C
C     REDUCE  X
C
      CALL DQRDC(X,LDX,N,P,QRAUX,JPVT,WORK,1)
C
C     DETERMINE WHICH COLUMNS TO USE.
C

      K = 0
      M = MINO(N,P)
      DO 20 KK=1,M
         IF (DABS(X(KK,KK)) .LE. TOL*DABS(X(1,1))) GO TO 30
         K = KK
   20 CONTINUE
   30 CONTINUE
C
C     SOLVE THE TRUNCATED LEAST SQUARES PROBLEM.
C
      IF (K .NE. 0)
     1   CALL DQRSL(X,LDX,N,K,QRAUX,Y,RSD,RSD,B,RSD,RSD,110,INFO)
C
C     SET THE UNUSED COMPONENTS OF B TO ZERO AND INITIALIZE JPVT
C     FOR UNSCRAMBLING
C
      DO 40  J=1,P
         JPVT(J) = -JPVT(J)
         IF (J .GT. K) B(J) = 0.
   40 CONTINUE
C
C     UNSCRAMBLE THE SOLUTION.
C
      DO 70  J=1,P
         IF (JPVT(J) .GT. 0) GO TO 70
         K = -JPVT(J)
         JPVT(J) =  K
   50    CONTINUE
            IF (K .EQ. J) GO TO 60
            T =  B(J)
            B(J) = B(K)
            B(K) = T
            JPVT(K) = -JPVT(K)
            K = JPVT(K)
         GO TO 50
   60    CONTINUE
   70 CONTINUE
      RETURN
      END
